/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AMO Localiser.
 *
 * The Initial Developer of the Original Code is
 * Andrew Williamson <eviljeff@eviljeff.com>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
function AMOLocaliser() {
	var Cc=Components.classes;
	var Ci=Components.interfaces;
	var that=this;
	
	this.preferenceService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
	this.preferenceService2 = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);

	const AMOLOCALES=new Array({lc:"cs",   reg: /(^|\/)cs[-]?[\w]{0,3}\//g},	 // * cs - Czech 
							   {lc:"da",   reg: /(^|\/)da[-]?[\w]{0,3}\//g},    // * da - Danish 
							   {lc:"de",   reg: /(^|\/)de[-]?[\w]{0,3}\//g},    // * de - German 
							   {lc:"en-US",reg: /(^|\/)en[-]?[\w]{0,2}\//g},    // * en-US - English (US)
							   {lc:"el"   ,reg: /(^|\/)el[-]?[\w]{0,3}\//g},    // * el - Greek 
							   {lc:"es-ES",reg: /(^|\/)es[-]?[\w]{0,2}\//g},    // * es-ES - Spanish 
							   {lc:"eu"   ,reg: /(^|\/)eu[-]?[\w]{0,3}\//g},    // * eu - Basque 
							   {lc:"fi"   ,reg: /(^|\/)fi[-]?[\w]{0,3}\//g},    // * fi - Finnish 
							   {lc:"fr"   ,reg: /(^|\/)fr[-]?[\w]{0,3}\//g},    // * fr - French 
							   {lc:"it"   ,reg: /(^|\/)it[-]?[\w]{0,3}\//g},    // * it - Italian 
							   {lc:"ja"   ,reg: /(^|\/)ja[-]?[\w]{0,3}\//g},    // * ja - Japanese 
							   {lc:"ko"   ,reg: /(^|\/)ko[-]?[\w]{0,3}\//g},    // * ko - Korean 
							   {lc:"mn"   ,reg: /(^|\/)mn[-]?[\w]{0,3}\//g},    // * mn - Mongolian 
							   {lc:"nl"   ,reg: /(^|\/)nl[-]?[\w]{0,3}\//g},    // * nl - Dutch 
							   {lc:"pl"   ,reg: /(^|\/)pl[-]?[\w]{0,3}\//g},    // * pl - Polish 
							   {lc:"pt-PT",reg: /(^|\/)pt-PT\//g},       		// * pt-PT - Portuguese 
							   {lc:"ro"   ,reg: /(^|\/)ro[-]?[\w]{0,3}\//g},    // * ro - Romanian 
							   {lc:"ru"   ,reg: /(^|\/)ru[-]?[\w]{0,3}\//g},    // * ru - Russian 
							   {lc:"sk"   ,reg: /(^|\/)sk[-]?[\w]{0,3}\//g},    // * sk - Slovak  
							   {lc:"sq"   ,reg: /(^|\/)sq[-]?[\w]{0,3}\//g},    // * sq - Albanian  
							   {lc:"uk"   ,reg: /(^|\/)uk[-]?[\w]{0,3}\//g},    // * uk - Ukrainian  
							   {lc:"zh-CN",reg: /(^|\/)zh-CN\//g},       		// * zh-CN - Chinese (Simplified) 
							   {lc:"ca"   ,reg: /(^|\/)ca[-]?[\w]{0,3}\//g},    // * ca - Catalan, Valencian  
							   {lc:"he"   ,reg: /(^|\/)he[-]?[\w]{0,3}\//g},    // * he - Hebrew 
							   {lc:"pt-BR",reg: /(^|\/)pt-BR\//g},       		// * pt-BR - Portuguese (Brazil)  
							   {lc:"tr"   ,reg: /(^|\/)tr[-]?[\w]{0,3}\//g},    // * tr - Turkish 
							   {lc:"zh-TW",reg: /(^|\/)zh-TW\//g}     			// * zh-TW - Chinese (Traditional) 	
							   );						 

	const AMOFIELDS=new Array('VersionReleasenotes','AddonName','AddonHomepage','AddonSupportemail',
							  'AddonSupporturl','AddonSummary','AddonDescription','AddonEula',
							  'AddonPrivacypolicy','AddonDevelopercomments');
	
	var last_MetaData;
	
	init();
	
	function MetaData(path,contentsData) {
		this.path=path;
		contentsData=contentsData.split("\n");
		var contents=new Object();
		var index;
		for (var i=0;i<contentsData.length;i++) {
			if ((index=contentsData[i].indexOf("="))!=-1) contents[contentsData[i].substring(0,index)]=contentsData[i].substring(index+1);
		}
		this.contents=contents;
		
		this.toString=function(){
			var out="";
			for (c in contents) out+= (out==""?"":",")+c;
			return "path:"+path+","+"contents:"+out;
		}
	}
	
	function XPILocale(locale,uri) {this.locale=locale;this.uri=uri;
										this.toString=function(){return "locale:"+locale+","+"uri:"+uri;};}
		
	/* shortcut functions to get & set preferences. Only for simple type prefs on amoLocaliser branch.*/
	function getPref(pref) {
		var ps=that.preferenceService2.getBranch("amoLocaliser.");
		var type= ps.getPrefType(pref);
		if (type==ps.PREF_BOOL) return ps.getBoolPref(pref);
		if (type==ps.PREF_INT) return ps.getIntPref(pref);
		if (type==ps.PREF_STRING) return ps.getCharPref(pref);
		return null;
	};
	this.getPref=getPref;

	function setPref(pref,value) {
		var ps=that.preferenceService2.getBranch("amoLocaliser.");
		var type= ps.getPrefType(pref);
		if (type==ps.PREF_BOOL) return ps.setBoolPref(pref,value);
		if (type==ps.PREF_INT) return ps.setIntPref(pref,value);
		if (type==ps.PREF_STRING) return ps.setCharPref(pref,value);
		return null;
	};
	this.setPref=setPref;
	
	function init(){
		document.getElementById("appcontent").addEventListener("DOMContentLoaded", addButtonToPage, true);
		document.getElementById("appcontent").addEventListener("unload", removeButtonFromPage, true);
	}
	this.init=init;
	
	function start(func) {
		switch (func) {
			case "xpi":
				var xpi=getFileFromDialog(Ci.nsIFilePicker.modeOpen,null,new Array({title:"Add-On File",filter:"*.xpi; *.jar"}));
				if (!xpi) return;
				last_MetaData=loadXPI(xpi);
				break;
			case "folder":
				var folder=getFileFromDialog(Ci.nsIFilePicker.modeGetFolder,null,null);
				if (!folder) return;
				last_MetaData=loadFolder(folder);
				break;
			case "apply":
				break;
			default:return;
		}
		dump(last_MetaData);
		linkUpData(last_MetaData);
		dump("\n***************************\n");
		document.getElementById("menu_amoLocaliserApply").removeAttribute("disabled");
		if (getPref('debug')) Application.console.open();
		
		//var notificationBox = getBrowser().getNotificationBox();
		/*var buttons = [{
          label: "undo",
          accessKey: "u",
          callback: function() {  }
		}];*/
	    /*notification=notificationBox.appendNotification(document.getElementById('menu_amoLocaliser').getAttribute('label'),
										"amoLocaliserNotification",
                                        "chrome://amoLocaliser/skin/icon.png",
                                        notificationBox.PRIORITY_INFO_LOW, null);*/
		
		//BrowserReload();
		//setTimeout(new function() {dump("$$$");getBrowser().getNotificationBox().removeAllNotifications();} ,1000);
		//notificationBox.removeNotification(notification);
	}
	this.start=start;
	
	function handleWebPageClick(aEvent) {
		if (aEvent.originalTarget.tagName.toLowerCase()!="button") return false;
		safelog(aEvent.originalTarget.getAttribute('id'));
		start(aEvent.originalTarget.getAttribute('function'));
		if (last_MetaData) aEvent.originalTarget.ownerDocument.getElementById("AMOLocaliserButton-Apply").removeAttribute("disabled");
		return false;
	}
	
	function loadXPI(xpi) {
		var zReader = Cc["@mozilla.org/libjar/zip-reader;1"].createInstance(Ci.nsIZipReader);
		zReader.open(xpi);
		
		var localeCount=0;
		var metaData=new Array();
		for (var enumr=zReader.findEntries("*"+getPref('properties')+"$");enumr.hasMore();) {
			var entry=enumr.getNext();
			//dump(entry+"\n");
			metaData.push(new MetaData(entry,readStringFromZipEntry(zReader,entry)));
			localeCount++;
		}
		safelog((localeCount==0?"No":localeCount)+" "+getPref('properties')+" files found");
		
		var chromeManifest=readStringFromZipEntry(zReader,"chrome.manifest");
		//dump(chromeManifest+"\n");
		
		var contentInf =/^\s*content\s+(\S+)\s+(\S+)\s*$/mg.exec(chromeManifest);
		if (contentInf) safelog("packageName: "+contentInf[1]+"; chromeUri: "+contentInf[2]);
		//else dump(contentInf+"\n");
		/*
		var xpiLocales=new Array();
		var localeMatch;
		var ree='^\\s*locale\\s+'+contentInf[1]+'\\s+(\\S+)\\s+(\\S+)\\s*$';
		var regex=new RegExp(ree,"mg");
		dump(">>"+ree+"\n");
		while ((localeMatch = regex.exec(chromeManifest)) != null)
		{
			xpiLocales.push(new XPILocale(localeMatch[1],localeMatch[2]));
  			//safelog("Found " + localeMatch[0] + ".  ");
		}
		dump(">>"+xpiLocales+"\n");*/
		
		/*var packageName=/\s*^content\s+(\S+)\s+\S+\s*$/mg.exec(chromeManifest);
		dump(">>"+packageName+"\n");
		var packageUri =/\s*^content\s+\S+\s+(\S+)\s*$/mg.exec(chromeManifest);
		dump(">>"+packageUri +"\n");*/
		
		zReader.close();
		return metaData;	
	}
	this.loadXPI=loadXPI;
	
	function readStringFromZipEntry(zipReader,entry) {
		if (!zipReader.hasEntry(entry)) return "";
		var zstream=zipReader.getInputStream(entry);
		var st=readStringFromIStream(zstream);
		zstream.close();
		return st;
	}
	
	function loadFolder(folder) {
		var localeCount=0;
		var metaData=new Array();
		var matches=glob(folder,new RegExp(getPref('properties').replace('.','\\.')+"$"));
		//extract matches and file contents, etc
		for (var i=0;i<matches.length;i++) {
			var entry=matches[i];
			metaData.push(new MetaData(entry.path.substr(folder.path.length+1).replace(/\\/g,'/'),readStringFromFile(entry)));
			localeCount++;
		}
		safelog((localeCount==0?"No":localeCount)+" "+getPref('properties')+" files found");
		return metaData;
	}
	
	function readStringFromFile(file) {
		var fstream = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
		fstream.init(file, -1, 0, 0);
		var st=readStringFromIStream(fstream);
		fstream.close();
		return st;
	}
	
	function linkUpData(metaData) {
		for (var i=0;i<AMOLOCALES.length;i++) {
			for (var j=0;j<metaData.length;j++) {
				if (AMOLOCALES[i].reg.test(metaData[j].path)) {
					insertData(AMOLOCALES[i].lc,metaData[j].contents);
					//safelog("got a match: "+AMOLOCALES[i].lc+" vs "+metaData[j].path);
					break;
				}//else safelog("no match: "+AMOLOCALES[i].lc+" vs "+metaData[j].path);
			}
		}
	}
	
	function insertData(localeCode,properties) {
		localeCode=localeCode.replace('-','_');
		var prefTree=Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("amoLocaliser.properties.");
		
		for (var i=0;i<AMOFIELDS.length;i++) {
			var f=AMOFIELDS[i];
			var input;
			if (content.document.getElementById(f+'_'+localeCode)) {
				// normal pages
				input=content.document.getElementById(f+'_'+localeCode);
			}
			else if (content.document.getElementById(f+'][') && content.document.getElementById('locale_'+localeCode)) {
				// malformed addon details page - its developer boxes don't have unique ids
				var inputs=content.document.getElementById('locale_'+localeCode).getElementsByTagName('textarea');
				input=(inputs)? inputs[0] : null;
			}
			
			if (!input) {
				// else safelog(f+'_'+localeCode+' not found on page');
				break;
			}
			if (properties[prefTree.getCharPref(f)]) {
				input.value=properties[prefTree.getCharPref(f)].replace(/\\n/g,'\n');
			}
			else safelog(prefTree.getCharPref(f)+' not in ('+localeCode+') properties file');
		}
	}
	
	function getFileFromString(fp) {
		var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
		file.initWithPath(fp);
		return file;
	}
	
	
	function readStringFromIStream(stream) {
		var is = Cc["@mozilla.org/intl/converter-input-stream;1"].createInstance(Ci.nsIConverterInputStream);
		var str="";
		try	{
			  is.init(stream, "UTF-8", 1024, Ci.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
			  var strr = {};
			  while (is.readString(4096, strr) != 0) str+=strr.value;
      	}
  		catch (e) {safelog("Error: failed reading from stream:\n" + e );}
		is.close();
		return str;
	}
	
	function getFileFromDialog(mode,filters,customFilters,suggestedFileName) {
		var fp = Cc["@mozilla.org/filepicker;1"].createInstance(Ci.nsIFilePicker);
		fp.init(window,null, mode);
		if (filters) fp.appendFilters(filters);
		if (suggestedFileName) fp.defaultString=suggestedFileName;
		if (customFilters) {
			for(var i=0;i<customFilters.length;i++) fp.appendFilter(customFilters[i].title,customFilters[i].filter);
		}
		if (fp.show() != Ci.nsIFilePicker.returnCancel) return fp.file;
		else return null;
	}
	
	function glob(folder,pattern) {
		if (!folder.exists() || !folder.isDirectory()) return new Array(0);
		var enumerator = folder.directoryEntries;
		var matches=new Array();
		while (enumerator.hasMoreElements()) {
  			var file = enumerator.getNext().QueryInterface(Ci.nsILocalFile);
			if (file.isDirectory()) matches=matches.concat(glob(file,pattern));
  			else if (pattern.test(file.leafName)) matches.push(file);
		}
		return matches;
	}
	
	function addButtonToPage(aEvent) {
		var doc = aEvent.originalTarget;
		var target=doc.getElementById('translationBox');
		if (!target) return;
		var div = doc.createElement("div");
		div.setAttribute("id","AMOLocaliserDiv");
		div.style.border='1px dashed blue';
		div.style.padding='5px';
		var tt=(last_MetaData!=null);
		div.appendChild(doc.createTextNode(document.getElementById('menu_amoLocaliser').getAttribute('label')+": "));
		div.appendChild(createDOMElement(doc,"button",new Array(
						  {name:"id",value:"AMOLocaliserButton-Apply"},{name:(tt?"disabled1":"disabled"),value:"true"},
						  {name:"function",value:"apply"},{name:'type',value:'button'}),
						  doc.createTextNode(document.getElementById('menu_amoLocaliserApply').getAttribute('label'))));
		div.appendChild(doc.createTextNode(" | "));
		div.appendChild(createDOMElement(doc,"button",new Array(
						  {name:"id",value:"AMOLocaliserButton-XPIBrowse"},{name:"function",value:"xpi"},
						  {name:'type',value:'button'}),
						  doc.createTextNode(document.getElementById('menu_amoLocaliserXPIBrowse').getAttribute('label'))));
		div.appendChild(createDOMElement(doc,"button",new Array(
						  {name:"id",value:"AMOLocaliserButton-FolderBrowse"},{name:"function",value:"folder"},
						  {name:'type',value:'button'}),
						  doc.createTextNode(document.getElementById('menu_amoLocaliserFolderBrowse').getAttribute('label'))));

		div.addEventListener("click", handleWebPageClick, true);
		target.insertBefore(div,doc.getElementById('locale-box'));
		safelog("adding button to page");
	}
	
	function removeButtonFromPage(aEvent) {
		var doc = aEvent.originalTarget;
		var target=doc.getElementById('AMOLocaliserDiv');
		if (!target) return;
		target.removeEventListener("click", handleWebPageClick, true);
		safelog("remove button listener");
	}
	
	function createDOMElement(doc,tagName,attributes,child) {
		var e=doc.createElement(tagName);
		for (var i=0;i<attributes.length;i++) e.setAttribute(attributes[i].name, attributes[i].value);
		if (child) e.appendChild(child);
		return e;
	}
	
	function createBlankProperties() {
		var out   = {value: false};
		window.openDialog('chrome://amolocaliser/content/propertiesEditor.xul','','chrome,resizable,dialog,modal',AMOFIELDS,out);
		//safelog(out.value);
		if (!out || !out.value) return;
		var file=getFileFromDialog(Ci.nsIFilePicker.modeSave,null,[{title:"Properties",filter:"*.properties"}],getPref("properties"))
	    if (!file) return;
		createPropertiesFile(file,AMOFIELDS,out.value);
	}
	this.createBlankProperties=createBlankProperties;
	
	function createPropertiesFile(file,properties,values) {
		var stream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
		var os = Cc["@mozilla.org/intl/converter-output-stream;1"].createInstance(Ci.nsIConverterOutputStream);
		var prefTree=Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch("amoLocaliser.properties.");
		try	{
		  stream.init(file, -1, -1, 0);
		  os.init(stream, "UTF-8", 4096, 0x0000);
		  for (var i=0;i<properties.length;i++) {
			os.writeString(prefTree.getCharPref(properties[i])+"="+values[i]+"\n");
		  }
		}
  		catch (e) {safelog("Error: failed writing to file:\n" + e );}
		os.close();
		stream.close();
	}
	
	function safelog(o) {
		if (!getPref("debug")) return;
		Application.console.log("AMOLocaliser: "+o);
	}
	
	function safedump(o) {
		if (!getPref("debug")) return;
		Application.console.log("AMOLocaliser: "+o);
	}
};

var amoLocaliser=new AMOLocaliser;